@@ -9,27 +9,11 @@ from typing import List, Optional
 
 from pydantic import BaseModel
 
-from .models import NetworkInterface, Payment
+from .models import Flavor, NetworkInterface, Payment
 
 LATEST_API_VERSION = 3
 
 
-class TokenEnable:
-    """Deprecated: Use TokenAdd instead."""
-
-    url = "/token/{token}/enable"
-    method = "POST"
-
-    class Request(BaseModel):
-        currency: str
-        dollars: int
-        affiliate_token: Optional[str] = None
-
-    class Response(BaseModel):
-        token: str
-        payment: Payment
-
-
 class TokenAdd:
     url = "/token/{token}/add"
     method = "POST"
@@ -59,37 +43,43 @@ class ServerLaunch:
     method = "POST"
 
     class Request(BaseModel):
-        machine_id: str
         days: int
         flavor: str
         ssh_key: str
         operating_system: str
-        currency: Optional[str] = None
-        """Currency only needs to be set if not paying with a token."""
         region: Optional[str] = None
-        organization: Optional[str] = None
-        token: Optional[str] = None
+        """null is automatic, otherwise a string region slug."""
+        token: str
+        """Token to draw from when launching the server."""
         quote: bool = False
+        """Don't launch, get a quote on how much it would cost"""
         affiliate_token: Optional[str] = None
-        affiliate_amount: None = None
-        """Deprecated field"""
-        settlement_token: Optional[str] = None
-        """Deprecated field. Use token instead."""
+        hostname: str = ""
+        """Hostname to refer to your server by."""
+        autorenew: bool = False
+        """
+        Automatically renew the server with the token used, keeping it at 1 week
+        expiration.
+
+        BETA FEATURE!!!
+        """
 
     class Response(BaseModel):
         payment: Payment
+        """Deprecated, not needed when paying with token. Only used for quote."""
         expiration: int
         machine_id: str
-        operating_system: str
-        flavor: str
         network_interfaces: List[NetworkInterface] = []
+        """Deprecated, use ipv4/ipv6 from ServerInfo instead."""
         created_at: int = 0
         region: Optional[str] = None
-        latest_api_version: int = LATEST_API_VERSION
+        """Deprecated, use ServerInfo instead."""
         created: bool = False
         paid: bool = False
+        """Deprecated, not needed when paying with token."""
         warning: Optional[str] = None
-        txid: Optional[str] = None
+        flavor: str = ""
+        """Deprecated, use ServerInfo instead."""
 
 
 class ServerTopup:
@@ -97,26 +87,19 @@ class ServerTopup:
     method = "POST"
 
     class Request(BaseModel):
-        machine_id: str
         days: int
-        token: Optional[str] = None
+        token: str
         quote: bool = False
-        currency: Optional[str] = None
-        """Currency only needs to be set if not paying with a token."""
         affiliate_token: Optional[str] = None
-        affiliate_amount: None = None
-        """Deprecated field"""
-        settlement_token: Optional[str] = None
-        """Deprecated field. Use token instead."""
 
     class Response(BaseModel):
         machine_id: str
         payment: Payment
+        """Deprecated, not needed when paying with token."""
         expiration: int
         paid: bool = False
+        """Deprecated, not needed when paying with token."""
         warning: Optional[str] = None
-        txid: Optional[str] = None
-        latest_api_version: int = LATEST_API_VERSION
 
 
 class ServerInfo:
@@ -128,8 +111,16 @@ class ServerInfo:
         expiration: int
         running: bool
         machine_id: str
-        network_interfaces: List[NetworkInterface]
+        ipv4: str
+        ipv6: str
         region: str
+        flavor: Flavor
+        deleted: bool
+        network_interfaces: List[NetworkInterface]
+        """Deprecated, use ipv4/ipv6 instead."""
+        operating_system: str
+        hostname: str
+        autorenew: bool
 
 
 class ServerStart:
@@ -142,11 +133,46 @@ class ServerStop:
     method = "POST"
 
 
+# Deprecated in favor of ServerDestroy
 class ServerDelete:
     url = "/server/{machine_id}/delete"
     method = "POST"
 
 
+class ServerDestroy:
+    url = "/server/{machine_id}/destroy"
+    method = "POST"
+
+
+class ServerForget:
+    url = "/server/{machine_id}/forget"
+    method = "POST"
+
+
 class ServerRebuild:
     url = "/server/{machine_id}/rebuild"
     method = "POST"
+
+
+class ServersLaunchedFromToken:
+    url = "/token/{token}/servers"
+    method = "GET"
+
+    class Response(BaseModel):
+        servers: List[ServerInfo.Response]
+
+
+class Flavors:
+    url = "/flavors"
+    method = "GET"
+
+    class Response(BaseModel):
+        flavors: dict[str, Flavor]
+
+
+class OperatingSystems:
+    url = "/operatingsystems"
+    method = "GET"
+
+    class Response(BaseModel):
+        operating_systems: list[str]
